;
;  KEY12.ASM
;  ---------
;	KEYPAD routines for Axiom HC12 development boards.
;     912x or CML12S KEYPAD uses Port H.
;
;  Variables - calling program must reserve these:
;  -----------
;     Global variables
;  	OLDKEY	1 byte, stores the previous key value for debouncing
;
;     Local variables
;	CNT1	1 byte, temporary debounce counter
;
;  Subroutines:
;  -----------
;  INITKEY: Set up keypad port
;  KEYSCAN:	Do a quick scan of the KEYPAD. Return A=key if pressed, 0 if not
;  KEYDLY:	Delay, wait for lines to settle

; Define Key Character Table

_KEY_TBL:  FCC  '*7410852#963DCBA'


; Initialize KEYPAD
; Port H 0-3 must be output
;        4-7 must be input with pull-down enabled

; Initialize KEYPAD
; Port H 0-3 must be output
;        4-7 must be input
INITKEY:
;	ldaa	#$0F	; set Port H direction (0=input, 1=output)
        ldaa	#$F0	; set Port H direction (0=input, 1=output)
	staa	DDRP	;

;        ldaa   #$f0      ; 
        ldaa   #$0f      ; 
        staa   PPSP      ; set for pull downs

;        ldaa   #$f0
         ldaa   #$0f
        staa   PERP      ; enable pull downs
        rts


;-------
GETKEY:
;	jsr	KEYWAIT		; wait for any keys to be released
	jsr	KEYSCAN		; scan keypad for new key
	tsta			; key pressed?
	beq	GETKEY		; loop if no
	rts



; Scan KEYPAD for input. return A=0 no input else A=key
; columns are active low, rows are then read active low
KEYSCAN:
;      movb  #$0F,PTP  ; set all KEYPAD columns
       movb  #$F0,PTP  ; set all KEYPAD columns
	ldaa	PTP		; get KEYPAD port
;	anda	#$F0		; key mask
        anda	#$0F		; key mask

	beq	KEYRESET	; branch if NO key pressed
;-------
; here if a key is pressed, determine which one
KEYSCAN1:
      clr    PTP      ; clear all KEYPAD columns
	clrb			; clear row counter
;      movb  #$01,PTP  ; set column 1
       movb  #$10,PTP  ; set column 1
KEYLP:
	jsr	KEYDLY		; wait for lines to settle
	ldaa	PTP		; get KEYPAD port
;	anda	#$F0		; mask row for data
        anda	#$0F		; mask row for data

; added for C32
        lsla
        lsla
        lsla
        lsla
        cmpa    #$00

	bne	KEYSCAN2	; branch if a key pressed, find row

; added for C32
        ldaa    PTP
        anda    #$f0
        lsla
        staa    PTP


;        lsl    PTP  	        ; set next column
	addb	#$04		; next row count
	cmpb	#$10		; check if no key found	
	bne	KEYLP		; loop for key
;-------
KEYRESET:
	clr	OLDKEY	; clear any old key
KEYRST1:
;	movb	#$10,CNT1	; set debounce time counter
        movb	#$80,CNT1	; set debounce time counter

KEYRST2:
;	movb	#$0F,PTP	; set all KEYPAD columns HIGH
        movb	#$F0,PTP	; set all KEYPAD columns HIGH
	clra			; clear key status
	rts

; here if a key is pressed, B=column count, A=row value
KEYSCAN2:
	incb			; increment key number
	lsla			; shift row value to low nibble


	bcc	KEYSCAN2	; loop until key found

; key found, check old or new...
	cmpb	OLDKEY	; test if same key
	bne	KEYNEW	; if not, start new key		
	dec	CNT1		; same key, debounce
	beq	KEYSAVE	; if yes, submit key
	bpl	KEYRST2	; if still in debounce, wait
; key serviced, wait for release...
	inc	CNT1		; prep for next cycle
	bra	KEYRST2	; 
KEYNEW:
	stab	OLDKEY	; save current key
	bra	KEYRST1	; start debounce next time
KEYSAVE:
	ldx	#_KEY_TBL	; get start of key table
	decb			; 0 offset key number
	ldaa	B,X		; load offset key value in a
	rts			; same key pressed, already said so, return it

;-------
KEYDLY:
	ldx	#$200		; clear offset count
KEYDLY1:
	nop
	dex 			; decrement counter
	bne	KEYDLY1	; loop if time not yet up
	rts


